package com.huimv.env.common.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.env.common.dao.entity.EnvDeviceRegisterEntity;
import com.huimv.env.common.entity.EnvElectricityDay;
import com.huimv.env.common.entity.EnvElectricityMonth;
import com.huimv.env.common.entity.EnvWaterDay;
import com.huimv.env.common.mapper.EnvElectricityDayMapper;
import com.huimv.env.common.mapper.EnvElectricityMonthMapper;
import com.huimv.env.common.service.EnvElectricityDayService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.env.common.utils.Const;
import com.huimv.env.common.utils.DateUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.sql.Date;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-10-21
 */
@Service
public class EnvElectricityDayServiceImpl extends ServiceImpl<EnvElectricityDayMapper, EnvElectricityDay> implements EnvElectricityDayService {
    @Resource
    private EnvElectricityDayMapper envElectricityDayMapper;
    @Resource
    private EnvElectricityMonthMapper envElectricityMonthMapper;

    @Override
    public EnvElectricityDay getDayElectricityConsumption(String farmCode, String deviceCode, String sensorSn, Date todayDate) {
        QueryWrapper<EnvElectricityDay> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("device_code", deviceCode);
//        queryWrapper.eq("sensor_sn",sensorSn);
        queryWrapper.eq("add_date", todayDate);
        return envElectricityDayMapper.selectOne(queryWrapper);
    }

    @Override
    public Result getTodayElectricity(Map<String, String> paramsMap) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        String farmCode = paramsMap.get("farmCode");
        String deviceCode = paramsMap.get("deviceCode");
        QueryWrapper<EnvElectricityDay> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("device_code", deviceCode);
        queryWrapper.eq("add_date", dateUtil.getTodayDate());
        EnvElectricityDay envElectricityDay = envElectricityDayMapper.selectOne(queryWrapper);
        if(envElectricityDay == null){
            return new Result(Const.ERR_TODAY_ELECTRICITY_CODE,Const.ERR_TODAY_ELECTRICITY_INFO,false);
        }else{
            return new Result(ResultCode.SUCCESS,envElectricityDay);
        }
    }

    @Override
    public Result getElectricityCount(Map<String, String> paramsMap) throws ParseException {
        //今日用电
        EnvElectricityDay envElectricityDay = _getDayElectricityCount(paramsMap);
        BigDecimal todayCountBd = new BigDecimal(0);
        BigDecimal todayLinkrelativeRatioBd = new BigDecimal(0);
        if(envElectricityDay != null){
            todayCountBd = envElectricityDay.getDayElectricity();
            todayLinkrelativeRatioBd = envElectricityDay.getLinkRelativeRatio();
        }

        //本月用电
        EnvElectricityMonth envElectricityMonth = _getMonthElectricityCount(paramsMap);
        BigDecimal monthCountBd = new BigDecimal(0);
        BigDecimal monthLinkrelativeRatioBd = new BigDecimal(0);
        if(envElectricityMonth != null){
            monthCountBd = envElectricityMonth.getMonthElectricity();
            monthLinkrelativeRatioBd = envElectricityMonth.getLinkRelativeRatio();
        }

        JSONObject resultJo = new JSONObject();
        resultJo.put("todayCount",todayCountBd);
        resultJo.put("todayLinkrelativeRatio",todayLinkrelativeRatioBd);
        resultJo.put("monthCount",monthCountBd);
        resultJo.put("monthLinkrelativeRatio",monthLinkrelativeRatioBd);
        return new Result(ResultCode.SUCCESS,resultJo);
    }

    //本月用电
    private EnvElectricityMonth _getMonthElectricityCount(Map<String, String> paramsMap) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        String farmCode = paramsMap.get("farmCode");
        QueryWrapper<EnvElectricityMonth> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("year", dateUtil.getThisYear());
        queryWrapper.eq("month", dateUtil.getThisMonth());
        return envElectricityMonthMapper.selectOne(queryWrapper);
    }

    //今日用电
    private EnvElectricityDay _getDayElectricityCount(Map<String, String> paramsMap) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        String farmCode = paramsMap.get("farmCode");
        QueryWrapper<EnvElectricityDay> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("add_date", dateUtil.getTodayDate());
        return envElectricityDayMapper.selectOne(queryWrapper);
    }

    @Override
    public void saveDayElectricity(JSONObject dataJo, EnvDeviceRegisterEntity envDeviceRegisterEntity, Timestamp nowTimestamp, Date todayDate, String farmCode) {
        String newWaterValue = dataJo.getString("value");
        BigDecimal newElectricityBd = new BigDecimal(newWaterValue).divide(new BigDecimal("100"), 2, BigDecimal.ROUND_HALF_UP);
        String deviceCode = dataJo.getString("deviceCode");
        String sensorSn = dataJo.getString("sensorSn");
        //获取环比
        BigDecimal linkRelativeRatioBd = _getLinkRelativeRatio( newElectricityBd,  deviceCode,  sensorSn,  todayDate,  farmCode);
        EnvElectricityDay envElectricityDay = new EnvElectricityDay();
        envElectricityDay.setDeviceCode(dataJo.getString("deviceCode"));
        envElectricityDay.setStartTime(nowTimestamp);
        envElectricityDay.setStartElectricity(newElectricityBd);
        envElectricityDay.setEndTime(nowTimestamp);
        envElectricityDay.setEndElectricity(newElectricityBd);
        envElectricityDay.setDayElectricity(new BigDecimal(0));
        envElectricityDay.setDataUnit("立方米");
        envElectricityDay.setSensorSn(dataJo.getInteger("sensorSn"));
        envElectricityDay.setAddDate(todayDate);
        envElectricityDay.setPigpenId(envDeviceRegisterEntity.getPigpenId());
        envElectricityDay.setUnitId(envDeviceRegisterEntity.getUnitId());
        envElectricityDay.setLinkRelativeRatio(linkRelativeRatioBd);
        envElectricityDay.setFarmCode(farmCode);
        int rows = envElectricityDayMapper.insert(envElectricityDay);
        System.out.println("添加记录数="+rows);
    }

    @Override
    public void updateDayElectricity(EnvElectricityDay envElectricityDay, JSONObject dataJo, EnvDeviceRegisterEntity envDeviceRegisterEntity, Timestamp nowTimestamp, Date todayDate, String farmCode) {
        BigDecimal endElectricityBd = new BigDecimal(dataJo.getString("value"));
        BigDecimal newDayElectricityBd = (endElectricityBd.subtract(envElectricityDay.getStartElectricity())).divide(new BigDecimal("100"), 2, BigDecimal.ROUND_HALF_UP);
        String deviceCode = dataJo.getString("deviceCode");
        String sensorSn = dataJo.getString("sensorSn");
        BigDecimal linkRelativeRatioBd = _getLinkRelativeRatio( newDayElectricityBd,  deviceCode,  sensorSn,  todayDate,  farmCode);
        envElectricityDay.setEndTime(nowTimestamp);
        envElectricityDay.setEndElectricity(endElectricityBd);
        envElectricityDay.setDayElectricity(newDayElectricityBd);
        envElectricityDay.setLinkRelativeRatio(linkRelativeRatioBd);
        int rows = envElectricityDayMapper.updateById(envElectricityDay);
        System.out.println("编辑记录数="+rows);
    }

    //读取环比
    private BigDecimal _getLinkRelativeRatio(BigDecimal newElectricityBd, String deviceCode, String sensorSn, Date todayDate, String farmCode) {
        QueryWrapper<EnvElectricityDay> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("device_code", deviceCode);
//        queryWrapper.eq("sensor_sn",sensorSn);
        queryWrapper.ne("add_date", todayDate);
        queryWrapper.orderByDesc("id");
        queryWrapper.last("LIMIT 1");
        EnvElectricityDay lastEnvElectricityDay = envElectricityDayMapper.selectOne(queryWrapper);
        if(lastEnvElectricityDay == null){
            return new BigDecimal(0);
        }else{
            try {
                return (newElectricityBd.subtract(lastEnvElectricityDay.getDayElectricity())).divide(lastEnvElectricityDay.getDayElectricity()).multiply(new BigDecimal(100));
            }catch (Exception e) {
                return new BigDecimal(0);
            }
        }
    }
}
