package com.huimv.env.common.service.impl;

import com.huimv.env.common.dao.entity.EnvSensorRegisterEntity;
import com.huimv.env.common.dao.repo.EnvSensorRegisterEntityRepo;
import com.huimv.env.common.service.ISensorRegisterService;
import com.huimv.env.common.utils.MathUtil2;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.stereotype.Service;

import java.sql.Date;
import java.sql.Timestamp;
import java.util.List;
import java.util.Optional;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
@Slf4j
public class SensorRegisterServiceImpl implements ISensorRegisterService {
    @Autowired
    private EnvSensorRegisterEntityRepo envSensorRegisterEntityRepo;
    @Autowired
    private MathUtil2 mathUtil;

    @Override
    public boolean isExistSensorRegister(String deviceCode, int sensorSn, String farmCode, int sensorSort){
        //
        List<Object[]> objectList = envSensorRegisterEntityRepo.getByDeviceCodeAndSensorSnAndFarmCode(deviceCode,sensorSn,farmCode,sensorSort);
        Object[] sensorObjArray = objectList.get(0);
        if(Integer.parseInt(sensorObjArray[0].toString()) > 0){
            return true;
        }else{
            return false;
        }
    }

    /**
     * @Method      : updateSensorRegister
     * @Description : 更新传感器注册数据
     * @Params      : [dataJo, farmCode]
     * @Return      : com.huimv.common.utils.Result
     *
     * @Author      : ZhuoNing
     * @Date        : 2022/8/18
     * @Time        : 16:25
     */
    @Override
    public void updateSensorRegister(String deviceCode, int sensorSn, Timestamp nowTimestamp, Date todayDate, String value, String farmCode, int sensorSort){
        EnvSensorRegisterEntity envSensorRegisterEntity = new EnvSensorRegisterEntity();
        envSensorRegisterEntity.setDeviceCode(deviceCode);
        envSensorRegisterEntity.setSensorSn(sensorSn);
        envSensorRegisterEntity.setFarmCode(farmCode);
        envSensorRegisterEntity.setSensorSort(sensorSort);
        Example<EnvSensorRegisterEntity> example = Example.of(envSensorRegisterEntity);
        Optional<EnvSensorRegisterEntity> optional = envSensorRegisterEntityRepo.findOne(example);
        if(!optional.isPresent()){
            log.error("该设备["+deviceCode+"]下的传感器["+sensorSn+"]未注册.");
        }else{
            EnvSensorRegisterEntity existSensorRegisterEntity = optional.get();
            existSensorRegisterEntity.setData(value);
            existSensorRegisterEntity.setLastTime(nowTimestamp);
            existSensorRegisterEntity.setLastDate(todayDate);
            envSensorRegisterEntityRepo.saveAndFlush(existSensorRegisterEntity);
        }
    }

    /**
     * @Method      : saveSensorRegister
     * @Description : 保存传感器注册数据
     * @Params      : [dataJo, nowTimestamp, todayDate, farmCode]
     * @Return      : com.huimv.common.utils.Result
     *
     * @Author      : ZhuoNing
     * @Date        : 2022/8/18
     * @Time        : 16:25
     */
    @Override
    public void saveSensorRegister(String deviceCode, int sensorSn, Timestamp nowTimestamp, Date todayDate, String value, String farmCode, int sensorSort){
        EnvSensorRegisterEntity envSensorRegisterEntity = new EnvSensorRegisterEntity();
        envSensorRegisterEntity.setDeviceCode(deviceCode);
        envSensorRegisterEntity.setSensorSn(sensorSn);
        envSensorRegisterEntity.setData(value);
        envSensorRegisterEntity.setFarmCode(farmCode);
        envSensorRegisterEntity.setLastTime(nowTimestamp);
        envSensorRegisterEntity.setLastDate(todayDate);
        envSensorRegisterEntity.setSensorSort(sensorSort);
        envSensorRegisterEntityRepo.saveAndFlush(envSensorRegisterEntity);
    }
}
