package com.huimv.env.manage.saas.service.impl;

import cn.hutool.core.convert.NumberChineseFormatter;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.env.manage.saas.dao.entity.BasePigpen;
import com.huimv.env.manage.mapper.BasePigpenMapper;
import com.huimv.env.manage.saas.service.IBasePigpenService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.env.manage.utils.Result;
import com.huimv.env.manage.utils.ResultCode;
import com.huimv.env.manage.utils.ResultUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2022-10-18
 */
@Service
public class BasePigpenServiceImpl extends ServiceImpl<BasePigpenMapper, BasePigpen> implements IBasePigpenService {



    @Autowired
    private BasePigpenMapper basePigpenMapper;

    @Override
    public Result listBasePigpen(Map<String, String> map, HttpServletRequest request) {
        String farmCode = map.get("farmCode");

        List<BasePigpen> basePigpenList = basePigpenMapper.selectList(new QueryWrapper<BasePigpen>().eq("farm_code", farmCode).eq("parent_id", 0));
        if (ObjectUtil.isEmpty(basePigpenList)){
            return new Result(10013,"没有栋舍",false);
        }
        List endList = new ArrayList();
        for (BasePigpen basePigpen : basePigpenList) {
            Map endMap = new HashMap();
            endMap.put("pigpenName",basePigpen.getPigpenName());
            endMap.put("id",basePigpen.getId());
            endMap.put("unitNum",basePigpenMapper.selectCount(new QueryWrapper<BasePigpen>().eq("parent_id",basePigpen.getId())));

            endList.add(endMap);
        }
        return new Result(ResultCode.SUCCESS,endList);
    }

    @Transactional
    @Override
    public Result saveBasePigpen(Map<String, String> map, HttpServletRequest request) {
        String farmCode = map.get("farmCode");

        String pigpenName = map.get("pigpenName");
        Integer unitNum = Integer.parseInt(map.get("unitNum"));

        BasePigpen basePigpen = new BasePigpen();
        basePigpen.setPigpenName(pigpenName);
        basePigpen.setParentId(0);
        basePigpen.setFarmCode(farmCode);
        if (  basePigpenMapper.exists(new QueryWrapper<BasePigpen>().eq("pigpen_name",pigpenName).eq("farm_code",farmCode))){
            return new Result(10001,"该栋舍已存在",false);
        }

        int insert = basePigpenMapper.insert(basePigpen);
        System.out.println(insert);
        for (Integer integer = 1; integer <= unitNum; integer++) {
            String format = NumberChineseFormatter.format(integer, false);

            BasePigpen basePigpenUnit = new BasePigpen();
            basePigpenUnit.setPigpenName(pigpenName+format+"单元");
            basePigpenUnit.setParentId(basePigpen.getId());
            basePigpenUnit.setFarmCode(farmCode);
            basePigpenMapper.insert(basePigpenUnit);
        }

        return new Result(10000,"添加成功",true);
    }

    @Override
    public Result listPigPenByFarmId(Map<String, String> map, HttpServletRequest request) {
        String farmCode = map.get("farmCode");
        List<BasePigpen> basePigpens = basePigpenMapper.selectList(new QueryWrapper<BasePigpen>().eq("farm_code", farmCode).eq("parent_id", 0));
        return ResultUtil.isNull(basePigpens);
    }

    @Override
    public Result listUnitByPigpenId(Map<String, String> map, HttpServletRequest request) {

        String pigpenId = map.get("pigpenId");
        List<BasePigpen> basePigpens = basePigpenMapper.selectList(new QueryWrapper<BasePigpen>().eq("parent_id", pigpenId));
        return ResultUtil.isNull(basePigpens);
    }
}
