package com.huimv.env.manage.saas.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.env.manage.saas.dao.entity.EnvDeviceEquipment;
import com.huimv.env.manage.mapper.EnvDeviceEquipmentMapper;
import com.huimv.env.manage.saas.service.IEnvDeviceEquipmentService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 环控绑定设备 服务实现类
 * </p>
 *
 * @author author
 * @since 2022-10-18
 */
@Service
public class EnvDeviceEquipmentServiceImpl extends ServiceImpl<EnvDeviceEquipmentMapper, EnvDeviceEquipment> implements IEnvDeviceEquipmentService {

    @Resource
    private EnvDeviceEquipmentMapper envDeviceEquipmentMapper;

    @Override
    public Result getEquipment(Map<String, String> paramsMap) {
        String farmCode = paramsMap.get("farmCode");
        String binding = paramsMap.get("binding");
        QueryWrapper<EnvDeviceEquipment> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq(StringUtils.isNotBlank(binding), "binding", binding);
        List<EnvDeviceEquipment> envDeviceEquipmentList = envDeviceEquipmentMapper.selectList(queryWrapper);
        return new Result(ResultCode.SUCCESS, envDeviceEquipmentList);
    }

    @Override
    public Result setEquipmentStatus(Map<String, String> paramsMap) {
        String farmCode = paramsMap.get("farmCode");
        String deviceCode = paramsMap.get("deviceCode");
        String port = paramsMap.get("port");
        String status = paramsMap.get("status");
        QueryWrapper<EnvDeviceEquipment> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("device_code", deviceCode);
        queryWrapper.eq("port", port);
        EnvDeviceEquipment envDeviceEquipment = envDeviceEquipmentMapper.selectOne(queryWrapper);
        envDeviceEquipment.setUploadStart(Integer.parseInt(status));
        int rows = envDeviceEquipmentMapper.updateById(envDeviceEquipment);
        if (rows == 0) {
            return new Result(ResultCode.FAIL);
        } else {
            return new Result(ResultCode.SUCCESS);
        }
    }

    @Override
    public Result getShowRoomEquipment(Map<String, String> paramsMap) {
        String unitId = paramsMap.get("unitId");
        String binding = paramsMap.get("binding");
        QueryWrapper<EnvDeviceEquipment> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("unit_id", unitId);
        queryWrapper.eq(StringUtils.isNotBlank(binding), "binding", binding);
        List<EnvDeviceEquipment> envDeviceEquipmentList = envDeviceEquipmentMapper.selectList(queryWrapper);
        return new Result(ResultCode.SUCCESS, envDeviceEquipmentList);
    }

    @Override
    public Result setShowRoomEquipmentStatus(Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        String status = paramsMap.get("status");
        EnvDeviceEquipment envDeviceEquipment = new EnvDeviceEquipment();
        envDeviceEquipment.setId(Integer.parseInt(id));
        envDeviceEquipment.setUploadStart(Integer.parseInt(status));
        int rows = envDeviceEquipmentMapper.updateById(envDeviceEquipment);
        if (rows == 0) {
            return new Result(ResultCode.FAIL);
        } else {
            return new Result(ResultCode.SUCCESS);
        }
    }
}
