package com.huimv.env.manage.saas.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.huimv.env.manage.entity.EnvDeviceRegister;
import com.huimv.env.manage.mapper.BasePigpenMapper;
import com.huimv.env.manage.mapper.LampConfigAllMapper;
import com.huimv.env.manage.saas.dao.entity.BasePigpen;
import com.huimv.env.manage.saas.dao.entity.LampConfig;
import com.huimv.env.manage.mapper.LampConfigMapper;
import com.huimv.env.manage.saas.dao.entity.LampConfigAll;
import com.huimv.env.manage.saas.service.IBasePigpenService;
import com.huimv.env.manage.saas.service.ILampConfigService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.env.manage.utils.Result;
import com.huimv.env.manage.utils.ResultCode;
import com.huimv.env.manage.utils.ResultUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2022-10-18
 */
@Service
public class LampConfigServiceImpl extends ServiceImpl<LampConfigMapper, LampConfig> implements ILampConfigService {

    @Autowired
    private LampConfigMapper lampConfigMapper;
    @Autowired
    private LampConfigAllMapper lampConfigAllMapper;

    @Autowired
    BasePigpenMapper basePigpenMapper;

    @Override
    public Result savelampConfig( LampConfig lampConfig, HttpServletRequest request) {
        String deviceId = lampConfig.getDeviceId();
        Integer unitId = lampConfig.getUnitId();
        deviceId = deviceId.substring(2);
        lampConfig.setDeviceId(deviceId);
        String location  = lampConfig.getLocation();
        if (lampConfigMapper.exists(new QueryWrapper<LampConfig>().eq("device_id",deviceId))){
            return new Result(10001,"保温灯已存在",false);
        }
        if (lampConfigMapper.exists(new QueryWrapper<LampConfig>().eq("unit_id",unitId).eq("location",location))){
            return new Result(10002,"该位置已有保温灯",false);
        }
        this.save(lampConfig);
        return new Result(10000,"添加成功",true);

    }

    @Override
    public Result lsitAllLamp(Map<String, String> map, HttpServletRequest request) {
        String farmCode = map.get("farmCode");
        List<LampConfig> lampConfigs = lampConfigMapper.selectList(new QueryWrapper<LampConfig>().eq("farm_code", farmCode));
        List endList = new ArrayList();
        for (LampConfig lampConfig : lampConfigs) {
            Map endMap = new HashMap();
            endMap.put("binding",lampConfig.getBinding());
            endMap.put("deviceId",lampConfig.getDeviceId());
            endMap.put("farmCode",lampConfig.getFarmCode());
            endMap.put("id",lampConfig.getId());
            endMap.put("unitId",lampConfig.getUnitId());
            endMap.put("lampName",lampConfig.getLampName());
            endMap.put("location",lampConfig.getLocation());
            BasePigpen basePigpen = basePigpenMapper.selectById(lampConfig.getUnitId());
            if (ObjectUtil.isNotEmpty(basePigpen)){
                endMap.put("pigpenName",basePigpen.getPigpenName());
            }else {
                endMap.put("pigpenName","该设备未绑定");
            }
            endList.add(endMap);
        }

        return ResultUtil.isNull(endList);
    }

    @Override
    public Result unbindingLamp(Map<String, String> map, HttpServletRequest request) {
        String deviceId = map.get("deviceId");
        LampConfig lampConfig = lampConfigMapper.selectOne(new QueryWrapper<LampConfig>().eq("device_id", deviceId));
        lampConfig.setLocation("");
        lampConfig.setBinding(0);
        lampConfig.setUnitId(0);
        lampConfig.setUploadStart(0);
        lampConfigMapper.updateById(lampConfig);
        return new Result(10000,"解绑成功",true);
    }

    @Override
    public Result bindingBaseLamp(Map<String, String> map, HttpServletRequest request) {
        String id = map.get("id");
        String unitId = map.get("unitId");
        String location = map.get("location");

        if (lampConfigMapper.exists(new QueryWrapper<LampConfig>().eq("unit_id",unitId).eq("location",location))){
            return new Result(10002,"该位置已有保温灯",false);
        }

        LampConfig lampConfig = new LampConfig();

        lampConfig.setId(Integer.parseInt(id));
        lampConfig.setUnitId(Integer.parseInt(unitId));
        lampConfig.setLocation(location);
        lampConfig.setBinding(1);

        lampConfigMapper.updateById(lampConfig);

        return new Result(10000,"绑定成功",true);
    }

    //根据日龄计算温度
    @Override
    public String updateTemp(String ageList, Integer dayAge) {

        String[] split = ageList.split(";");
        String temp = split[split.length-1].split(",")[1];
        for (int i = 0; i < split.length; i++) {
            String  lamp1 = split[i];
            if (dayAge<=Integer.parseInt(lamp1.split(",")[0])){
                temp = lamp1.split(",")[1];
                String tempStr = (Double.parseDouble(temp) * 10) + "";
                return  tempStr.substring(0,tempStr.lastIndexOf("."));
            }
        }

        String tempStr = (Double.parseDouble(temp) * 10) + "";

        return  tempStr.substring(0,tempStr.lastIndexOf("."));
    }

    @Override
    @Transactional
    public Result allToConfig(Map<String, Object> map, HttpServletRequest request) {
        try {
            Integer initDayAge =  (Integer) map.get("initDayAge");
            List<String> dayAgeList =  (List<String> )map.get("dayAgeList");
            //
            Integer boardGear =(Integer) map.get("boardGear");
            String unitId =(String) map.get("unitId");
            String id = (String)map.get("id");

            LampConfig lampConfig = new LampConfig();
            lampConfig.setInitAge(initDayAge);
            lampConfig.setBoardGear(boardGear);
            StringBuilder dayAgeListStr = new StringBuilder();
            for (String s : dayAgeList) {
                dayAgeListStr.append(s).append(";");
            }
            String temp = updateTemp(dayAgeListStr.toString(), initDayAge);
            lampConfig.setTempSet(temp);

            lampConfig.setAgeList(dayAgeListStr.toString());
            lampConfig.setDayAge(initDayAge);
            UpdateWrapper<LampConfig> wrapper = new UpdateWrapper<>();
            if (StringUtils.isBlank(id)){
                LampConfigAll lampConfigAll = new LampConfigAll();
                lampConfigAll.setBoardGear(boardGear);
                lampConfigAll.setInitDayage((initDayAge));
                lampConfigAll.setListDayage((dayAgeListStr.toString()));

                lampConfigAllMapper.update(lampConfigAll,new UpdateWrapper<LampConfigAll>().eq("unit_id",unitId));
                wrapper.eq("unit_id",unitId);
                lampConfigMapper.update(lampConfig,wrapper);
                return new Result(10000,"全局配置成功",true);
            }else {
                wrapper.eq("id",id);
                lampConfigMapper.update(lampConfig,wrapper);
                return new Result(10000,"配置成功",true);
            }

        }catch (Exception e){
            System.out.println("----------->"+e);
            return new Result(10001,"配置失败",true);
        }

    }

    @Override
    public Result listByUnit(Map<String, String> map, HttpServletRequest request) {
        String unitId = map.get("unitId");
        QueryWrapper<LampConfig> wrapper = new QueryWrapper<>();
        wrapper.eq("unit_id",unitId);
        List<LampConfig> lampConfigs = lampConfigMapper.selectList(wrapper);
        return ResultUtil.isNull(lampConfigs);
    }

    @Override
    public Result countByUnit(Map<String, String> map, HttpServletRequest request) {
        String unitId = map.get("unitId");
        QueryWrapper<LampConfig> wrapper = new QueryWrapper<>();
        wrapper.eq("unit_id",unitId);
        Long countAll = lampConfigMapper.selectCount(wrapper);
        wrapper.eq("upload_start",1);
        Long countStart = lampConfigMapper.selectCount(wrapper);
        Long countClose =countAll -countStart;
        Map endMap = new HashMap();
        endMap.put("countAll",countAll);
        endMap.put("countStart",countStart);
        endMap.put("countClose",countClose);
        if (countStart >0){
            endMap.put("start",true);
        }else {
            endMap.put("start",false);
        }

        return new Result(ResultCode.SUCCESS,endMap) ;
    }

    @Override
    public Result updateLamp(Map<String, Object> map, HttpServletRequest request) {
        Boolean uploadStart = (Boolean) map.get("uploadStart");
        String id = (String) map.get("id");
        String unitId = (String)map.get("unitId");
        LampConfig lampConfig = new LampConfig();
        if (uploadStart){
            lampConfig.setUploadStart(1);
        }else {
            lampConfig.setUploadStart(0);
        }


        if (ObjectUtil.isNotEmpty(unitId)){
            if (uploadStart  && lampConfigMapper.exists(new QueryWrapper<LampConfig>().isNull("temp_set").eq("unit_id",unitId))) {
                return new Result(10001,"有设备未配置，请先进行配置",false);

            }
            lampConfigMapper.update(lampConfig,new QueryWrapper<LampConfig>().eq("unit_id",unitId));
        }else {
            if (uploadStart  && lampConfigMapper.exists(new QueryWrapper<LampConfig>().isNull("temp_set").eq("id",id))) {
                return new Result(10001,"设备未配置，请先进行配置",false);

            }
            lampConfig.setId(Integer.parseInt(id));

            lampConfigMapper.updateById(lampConfig);
        }

        return new Result(10000,"配置成功",true);
    }

    @Override
    public Result listByLampId(Map<String, String> map, HttpServletRequest request) {
        String lampId = map.get("lampId");
        LampConfig lampConfig = lampConfigMapper.selectById(lampId);
        if (ObjectUtil.isEmpty(lampConfig)){
            return new Result(10001,"设备不存在",false);
        }

        Map endMap =new HashMap();
        endMap.put("initDayage",lampConfig.getInitAge());
        endMap.put("listDayage",lampConfig.getAgeList().split(";"));
        endMap.put("id",lampConfig.getId());
        endMap.put("lampName",lampConfig.getLampName());
        endMap.put("deviceId",lampConfig.getDeviceId());
        endMap.put("openStrart",lampConfig.getOpenStart());
        endMap.put("uploadStart",lampConfig.getUploadStart());
        endMap.put("boardGear",lampConfig.getBoardGear());
        endMap.put("tempSet",lampConfig.getTempSet());
        endMap.put("location",lampConfig.getLocation());
        endMap.put("dayage",lampConfig.getDayAge());
        endMap.put("unitId",lampConfig.getUnitId());
        endMap.put("farmCode",lampConfig.getFarmCode());
        endMap.put("binding",lampConfig.getBinding());
        return new Result(ResultCode.SUCCESS,endMap);
    }
}