package com.huimv.env.manage.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.huimv.env.manage.entity.EnvAmmonia;
import com.huimv.env.manage.entity.EnvHumi;
import com.huimv.env.manage.entity.EnvTemp;
import com.huimv.env.manage.entity.dto.TimeData;
import com.huimv.env.manage.mapper.EnvAmmoniaMapper;
import com.huimv.env.manage.mapper.EnvHumiMapper;
import com.huimv.env.manage.mapper.EnvTempMapper;
import com.huimv.env.manage.service.IEnvHumiService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.env.manage.utils.Result;
import com.huimv.env.manage.utils.ResultCode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2022-08-18
 */
@Service
public class EnvHumiServiceImpl extends ServiceImpl<EnvHumiMapper, EnvHumi> implements IEnvHumiService {
    @Autowired
    private EnvHumiMapper envHumiMapper;

    @Autowired
    private EnvTempMapper envTempMapper;

    @Autowired
    private EnvAmmoniaMapper envAmmoniaMapper;

    @Override
    public Result dataList(Map<String, String> paramMap) {
        String deviceCode = paramMap.get("deviceCode");
        String startTime = paramMap.get("startTime");
        String endTime = paramMap.get("endTime");
        if (StringUtils.isBlank(startTime)){
            startTime = DateUtil.format(DateUtil.beginOfDay(new Date()),"yyyy-MM-dd");
        }
        if (StringUtils.isBlank(endTime)){
            endTime = DateUtil.format(DateUtil.endOfDay(new Date()),"yyyy-MM-dd");
        }
        if(startTime.length() ==10){
            startTime = startTime+" 00:00:00";
        }else {
            startTime = startTime+"00:00:00";
        }
        if (endTime.length() ==10) {
            endTime = endTime+" 23:59:59";
        }else {
            endTime = endTime+"23:59:59";
        }



        Map<String,List> endMap  = new HashMap();
        QueryWrapper<EnvAmmonia> ammoniaWrapper = new QueryWrapper<EnvAmmonia>().eq("device_code", deviceCode).between("ask_time", startTime, endTime);
        List<TimeData> ammoinData  =  envAmmoniaMapper.listByTime(ammoniaWrapper);


        QueryWrapper<EnvHumi> humiWrapper = new QueryWrapper<EnvHumi>().eq("device_code", deviceCode).between("ask_time", startTime, endTime);
        List<TimeData> humiData  =  envHumiMapper.listByTime(humiWrapper);


        QueryWrapper<EnvTemp> tempWrapper = new QueryWrapper<EnvTemp>().eq("device_code", deviceCode).between("ask_time", startTime, endTime);
        List<TimeData> tempData  =  envTempMapper.listByTime(tempWrapper);

        int size = Math.min(tempData.size(), humiData.size());
        List<Map> list  = new ArrayList<>();
        for (int i = 0; i <size ; i++){
            Map map = new HashMap();
            TimeData humiData1 = humiData.get(i);
            TimeData tempData1 = tempData.get(i);
            map.put("hourTime",humiData1.getHourTime());
            map.put("humi",humiData1.getDataValue());
            map.put("temp",tempData1.getDataValue());
            list.add(map);
        }



        endMap.put("ammoin",ammoinData);
        endMap.put("humiAndtemp",list);
        return new Result(ResultCode.SUCCESS,endMap);
    }
}
